; Ported from a portion of my ancient directory lister v7.10.
; Relicensed and rereleased.

; MIT License
;
; Copyright (c) 2022 Jerome Shidel
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.

; NASM 2.13.03 or later

use16

cpu 8086

org 0x100

mov		di, AnsiQueryString
PrintLoop:
mov  		dl, [di] 		; Load Character
inc  		di			; Increment di pointer offset
cmp  		dl, 0			; Test for end of string
je   		PrintDone 		; If end of string, jump to check for result
mov  		ah, 0x02		; Function - Write Char to StdOut
int  		0x21		    	; Send Character to StdOut
jmp  		PrintLoop		; Print next character

PrintDone:

mov  		ah, 0x06		; Direct Console I/O
mov  		dl, 0xFF		; 0xff means get a character
int  		0x21			; Fetch 1st character
jz   		BadResponse		; if ZF, no character available
cmp  		al, 27			; Verify 1st character is ESC
jne  		BadResponse
mov  		ah, 0x06
mov  		dl, 0xFF
int  		0x21			; Fetch 2nd character
jz   		BadResponse
cmp 		al, '['			; Verify 2nd character is left brace
jne  		BadResponse

IgnoreLoop:				; Ignore all characters upto the letter R
mov  		ah, 0x06
mov  		dl, 0xFF
int  		0x21
jz   		BadResponse		; Ran out of letters before R was found
cmp  		al,'R'
jne  		IgnoreLoop		; Continue ignoring

BadResponse:
mov  		ah, 0x06
mov  		dl, 0xFF
int  		0x21			; Get one more character, that needs to be CR
jz   		DisplayCleanUp		; Test failed, clean up displayed garbage characters
cmp  		al,13
mov  		dx, AnsiGoodString
mov	 	al, 0
je   		Supported

DisplayCleanUp:
mov  		di, 	AnsiQueryString

CleanUpLoop:
mov  		dl, [di]
inc  		di
cmp  		dl, 0
je   		Unsupported
mov  		ah, 0x02
mov  		dl, 8			; backspace
int  		0x21
mov  		ah, 0x02		; space
mov  		dl, 32
int  		0x21
mov  		ah, 0x02
mov  		dl, 8			; another backspace
int  		0x21
jmp  		CleanUpLoop

Unsupported:
mov  		dx, AnsiBadString
mov		al, 1
Supported:
push 		dx
mov	 	ah, 0x09
mov  		dx, AnsiIsString
int	 	0x21			; Display Result String
pop  		dx
int	 	0x21			; Display Result String
mov	 	ah, 0x4c
int  		0x21			; terminate with result code 0/1

AnsiQueryString:
		db 		27,'[6n',0
AnsiIsString:
		db  		'TESTANSI=$'
AnsiGoodString:
		db		'TRUE$'
AnsiBadString:
		db		'FALSE$'


