; MIT License
;
; Copyright (c) 2022 Jerome Shidel
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.

; NASM 2.13.03 or later

use16

cpu 8086

%include 'NASMPLUS.INC'

org 0x100

ProbeVesa:
; Video BIOS - Get VESA SuperVGA information
; Returns AL=0x4f, if function is supported. AH=0x00 for success.
; Requires SuperVGA.
push		cs
pop		es
mov		di, VesaBuffer
mov		ax, 0x4f00
int		0x10
mov		dx, CardVesa
cmp		ax, 0x004f
je		Identified

ProbeVGA:
; Video BIOS - Video Refresh Control, Enable video refresh
; Returns AL=0x12, if function is supported. Requires VGA or better.
mov		ax, 0x1200
mov		bl, 0x36
int		0x10
mov		dx, CardVGA
cmp		al, 0x12
je		Identified

ProbeMCGA:
; Video BIOS - Video Addressing, Enable CPU video addressing
; Returns AL=0x12, if function is supported. Requires MCGA/VGA or better.
mov		ax, 0x1200
mov		bl, 0x32
int		0x10
mov		dx, CardMCGA
cmp		al, 0x12
je		Identified

ProbeEGA:
; Video BIOS - Get EGA Info. (AL does not need set, but will clear it anyway)
; If BH is unchanged on return, then EGA or later is not present.
mov		ax, 0x1200
mov		bx, 0xff10
int		0x10
mov		dx, CardEGA
cmp		bh, 0xff
jne		Identified

ProbeCGA:
; At this point, we are barely useful anymore. It's either CGA, MDA or one of
; the more rare cards in between. Just going to test if it is in color or mono.
; If color assume CGA, otherwise MDA.
mov		ax, 0x0040
push 		ax
pop		es
mov		si, 0x0063
mov		ax, [es:si]
mov		dx, CardCGA
cmp		ax, 0x3d4
je		Identified

ProbeMDA:
; Don't care anymore, assume its just MDA
mov		dx, CardMDA

Identified:
mov		ax, dx

; get address of prefix to drive letter and print it
push		ax
mov		ah, 0x09
mov     	dx, Message
int		0x21
pop		dx

; print Video Card type
int		0x21

; print tail of drive letter message and print it
mov     	dx, Last
int		0x21

; Return to DOS
; mov		ax, 0x4c00
; int		0x21
ret

Message:
		db 'VIDEO_KIND=$'
Last:
		db 0x0d, 0x0a, '$'
CardVesa:
		db 'SuperVGA$'
CardVGA:
		db 'VGA$'
CardMCGA:
		db 'MCGA$'
CardEGA:
		db 'EGA$'
CardCGA:
		db 'CGA$'
CardMDA:
		db 'MDA$'
CardUnknown:
		db 'unknown$'
VesaBuffer:
		; times 256 db 0
